// $Header: p:\\graphics\\CVS/src/GraphicsTown/TownViewWidget.cpp,v 1.12 2000/11/28 01:15:48 gleicher Exp $
#include "GrTown_PCH.H"

#include "TownViewWidget.H"
#include "DrawingState.H"
#include "GrWorld.H"
#include "GraphicsTownUI.H"
#include "GrObject.H"
#include "DrawUtils.H"
#include "Texture.H"
#include <time.h>
#include "FlyCamera.H"

// each idle time, do a UI step if need be...
void tvIdler(void* v)
{
  TownViewWidget* tv = (TownViewWidget*) v;
  if (tv->getCamera()->uiStep())
	tv->damage(1);

  unsigned long t = clock();
  for(GrObject* g = theObjects; g; g = g->next)
	g->simulateUntil(t);
  tv->damage(1);
}


TownViewWidget::TownViewWidget(int x, int y, int w, int h, 
							   const char* l)
  : Fl_Gl_Window(x,y,w,h,l)
{
  // we will probably want them all...
  mode(FL_RGB | FL_DOUBLE | FL_DEPTH | FL_ALPHA /*| FL_STENCIL*/);
  Fl::add_idle(tvIdler,this);
  followCamera = new FollowCam();
}
  
unsigned long lastDrawDone = 0;

void TownViewWidget::draw()
{
  // figure out how to draw
  DrawingState drst;
  getStateFromUI(&drst);
  glEnable(GL_TEXTURE_2D);

  if (drst.backCull) {
	  glEnable(GL_CULL_FACE);
	  glCullFace(GL_BACK);
  } else
	  glDisable(GL_CULL_FACE);

  glFrontFace(GL_CCW);

	// set up the camera for drawing!

  glEnable( GL_DEPTH_TEST );

  glMatrixMode(GL_PROJECTION);
  glViewport(0,0,w(),h());
  glLoadIdentity();

  // compute the aspect ratio so we don't distort things
  double aspect = ((double) w()) / ((double) h());
  gluPerspective(drst.fieldOfView, aspect, 1, 60000);

  // put the camera where we want it to be
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  setupLights(&drst);
  drst.camera->getTransform();
  drst.camera->setTransform();

  glClearStencil(0);

  // the actual clearing goes on in the sky routine since its the only
  // thing that knows what color to make the sky
  drawSky(&drst);
  drawEarth(&drst);

  GrObject* g;
  drawObList(theObjects,&drst);
  for(g = theObjects; g; g = g->next) {
	  if (g != drst.camera) {
		  glPushMatrix();
		  glMultMatrixf(&g->transform[0][0]);
		  g->drawAfter(&drst);
		  glPopMatrix();
	  }
  }

  if (lastDrawDone) {
	  double ifr = ((double)CLOCKS_PER_SEC) / (double) (clock()-lastDrawDone);
	  ui->rate->value(ifr);
  }
  lastDrawDone = clock();
}

void TownViewWidget::getStateFromUI(DrawingState* st)
{
	st->timeOfDay = (int) ui->time->value();
	st->fieldOfView = (float) ui->fov->value();
	st->camera = getCamera();
	st->backCull = (bool) ui->cull->value();
	st->drGrTex = (bool) ui->lgTex->value();
}

GrObject* TownViewWidget::getCamera()
{
  int p = ui->pickCamera->value();
  if (p) {
	if (ui->follower->value()) {
	  followCamera->following =  (GrObject*) ui->pickCamera->data(p);
	  return followCamera;
	}
	return (GrObject*) ui->pickCamera->data(p);
  } else
	return defaultCamera;
}

int TownViewWidget::handle(int e)
{
	switch(e) {
	case FL_SHOW:
		show();
		return 1;
	default:
		int r = getCamera()->handle(e);
		if (r) damage(1);
		return r;
	};
}

